@@ -38,7 +38,6 @@ class KeyboardGTK(gtk.DrawingArea):
         self.connect("enter-notify-event",   self._cb_mouse_enter)
         self.connect("leave-notify-event",   self._cb_mouse_leave)
         self.connect("configure-event",      self._cb_configure_event)
-        config.scanning_notify_add(self.reset_scan)
 
     def clean(self):
         self.stop_click_polling()
@@ -46,7 +45,7 @@ class KeyboardGTK(gtk.DrawingArea):
 
     def start_click_polling(self):
         self.stop_click_polling()
-        self.click_timer = gobject.timeout_add(20, self._cb_click_timer)
+        self.click_timer = gobject.timeout_add(1, self._cb_click_timer)
         self.click_detected = False
 
     def stop_click_polling(self):
@@ -173,7 +172,7 @@ class KeyboardGTK(gtk.DrawingArea):
         context.set_source_rgba(float(self.basePane.rgba[0]),
                     float(self.basePane.rgba[1]),
                     float(self.basePane.rgba[2]),
-                    float(self.basePane.rgba[3]))#get from .sok
+                    float(self.basePane.rgba[3]))#get from .onboard
         context.paint()
 
 
@@ -185,7 +184,7 @@ class KeyboardGTK(gtk.DrawingArea):
             context.set_source_rgba(float(self.activePane.rgba[0]),
                         float(self.activePane.rgba[1]),
                         float(self.activePane.rgba[2]),
-                        float(self.activePane.rgba[3]))#get from .sok
+                        float(self.activePane.rgba[3]))#get from .onboard
             context.fill()
             self.activePane.paint(context)
 
