@@ -24,7 +24,8 @@ from Onboard.KeyGtk import *
 from Onboard.Pane import Pane
 from Onboard.KbdWindow import KbdWindow, KbdPlugWindow
 from Onboard.KeyboardSVG import KeyboardSVG
-from Onboard.utils       import show_confirmation_dialog
+from Onboard.utils       import show_confirmation_dialog, CallOnce
+from Onboard.Appearance import Theme
 
 
 ### Config Singleton ###
@@ -78,17 +79,31 @@ class OnboardGtk(object):
 
         # load the initial layout
         self.update_layout()
-        config.layout_filename_notify_add(self.load_layout)
 
         # connect notifications for keyboard map and group changes
         self.keymap = gtk.gdk.keymap_get_default()
         self.keymap.connect("keys-changed", self.cb_keys_changed) # map changes
         gtk.gdk.event_handler_set(cb_any_event, self)          # group changes
 
+        # connect config notifications here to keep config from holding
+        # references to keyboard objects.
+        once = CallOnce(20).enqueue  # delay callbacks 50ms
+        theme_change  = lambda x: once(self.cb_theme_changed, x)
+        update_layout = lambda x: once(self.update_layout, True)
+        queue_draw    = lambda x: once(self.keyboard.queue_draw)
+
+        config.layout_filename_notify_add(update_layout)
+        config.color_scheme_filename_notify_add(update_layout)
+        config.key_label_overrides_notify_add(update_layout)
+        config.theme_attributes_notify_add(queue_draw)
+        config.snippets_notify_add(update_layout)
+
+        config.scanning_notify_add(lambda x: \
+                                     self.keyboard.reset_scan())
+
         # create status icon
         self.status_icon = Indicator(self._window)
-        if self.status_icon.is_appindicator():
-            self.status_icon.connect("quit-onboard", self.do_quit_onboard)
+        self.status_icon.connect("quit-onboard", self.do_quit_onboard)
 
         # Callbacks to use when icp or status icon is toggled
         config.show_status_icon_notify_add(self.show_hide_status_icon)
@@ -111,7 +126,15 @@ class OnboardGtk(object):
         # the user what to do
         if config.onboard_xembed_enabled:
             if not config.is_onboard_in_xembed_command_string():
-                question = _("Onboard is configured to appear with the dialog to unlock the screen; for example to dismiss the password-protected screensaver.\n\nHowever the system is not configured anymore to use onboard to unlock the screen. A possible reason can be that another application configured the system to use something else.\n\nWould you like to reconfigure the system to show onboard when unlocking the screen?")
+                question = _("Onboard is configured to appear with the dialog to "
+                             "unlock the screen; for example to dismiss the "
+                             "password-protected screensaver.\n\n"
+                             "However the system is not configured anymore to use "
+                             "Onboard to unlock the screen. A possible reason can "
+                             "be that another application configured the system to "
+                             "use something else.\n\n"
+                             "Would you like to reconfigure the system to show "
+                             "Onboard when unlocking the screen?")
                 reply = show_confirmation_dialog(question)
                 if reply == True:
                     config.onboard_xembed_enabled = True
@@ -121,7 +144,11 @@ class OnboardGtk(object):
                     config.onboard_xembed_enabled = False
             else:
                 if not config.gss_xembed_enabled:
-                    question = _("Onboard is configured to appear with the dialog to unlock the screen; for example to dismiss the password-protected screensaver.\n\nHowever this function is disabled in the system.\n\nWould you like to activate it?")
+                    question = _("Onboard is configured to appear with the dialog "
+                                 "to unlock the screen; for example to dismiss "
+                                 "the password-protected screensaver.\n\n"
+                                 "However this function is disabled in the system.\n\n"
+                                 "Would you like to activate it?")
                     reply = show_confirmation_dialog(question)
                     if reply == True:
                         config.onboard_xembed_enabled = True
@@ -141,7 +168,7 @@ class OnboardGtk(object):
     def show_hide_taskbar(self):
         """
         This method shows or hides the taskbard depending on whether there
-        is an alternative way to unminimize the onboard window.
+        is an alternative way to unminimize the Onboard window.
         This method should be called every time such an alternative way
         is activated or deactivated.
         """
@@ -190,7 +217,7 @@ class OnboardGtk(object):
     def cb_status_icon_clicked(self,widget):
         """
         Callback called when status icon clicked.
-        Toggles whether onboard window visibile or not.
+        Toggles whether Onboard window visibile or not.
 
         TODO would be nice if appeared to iconify to taskbar
         """
@@ -213,10 +240,16 @@ class OnboardGtk(object):
             return False
         return True
 
+    def cb_theme_changed(self, theme_filename):
+        # load and apply the theme
+        theme = Theme.load(theme_filename)
+        if theme:
+            theme.apply()
+
     def update_layout(self, force_update=False):
         """
         Checks if the X keyboard layout has changed and
-        (re)loads onboards layout accordingly.
+        (re)loads Onboards layout accordingly.
         """
         keyboard_state = (None, None)
 
@@ -235,17 +268,22 @@ class OnboardGtk(object):
 
         if self.keyboard_state != keyboard_state or force_update:
             self.keyboard_state = keyboard_state
-            self.load_layout(config.layout_filename)
+            self.load_layout(config.layout_filename, 
+                             config.color_scheme_filename)
 
         # if there is no X keyboard, poll until it appears
         if not vk and not self.vk_timer:
             self.vk_timer = gobject.timeout_add_seconds(1, self.cb_vk_timer)
 
-    def load_layout(self, filename):
-        _logger.info("Loading keyboard layout from " + filename)
+    def load_layout(self, layout_filename, color_scheme_filename):
+        _logger.info("Loading keyboard layout from " + layout_filename)
+        if (color_scheme_filename):
+            _logger.info("Loading color scheme from " + color_scheme_filename)
         if self.keyboard:
             self.keyboard.clean()
-        self.keyboard = KeyboardSVG(self.get_vk(), filename)
+        self.keyboard = KeyboardSVG(self.get_vk(), 
+                                    layout_filename, 
+                                    color_scheme_filename)
         self._window.set_keyboard(self.keyboard)
 
     def get_vk(self):
@@ -283,6 +321,7 @@ class OnboardGtk(object):
 
 
 def cb_any_event(event, onboard):
+    # Update layout on keyboard group changes
     # XkbStateNotify maps to gtk.gdk.NOTHING
     # https://bugzilla.gnome.org/show_bug.cgi?id=156948
     if event.type == gtk.gdk.NOTHING:
